/* 
    kleds for KDE2/KDE3

    Copyright (C) 2001 Hans Matzen (hans@hansmatzen.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "dockwidget.h"
#include <qtooltip.h>
#include <klocale.h>

dockwidget::dockwidget(QWidget *mainwin, QWidget *parent, 
		       const char *name) : QWidget(parent,name)
{
    // create the widget in the toolbar
    KWin::setSystemTrayWindowFor(winId(), mainwin->winId());
    
 
    // add tooltip
    QToolTip::add(this,"KLeds");
    

    num_label    = new QLabel (I18N_NOOP("N"),this); 
    num_led      = new KLed(this,"num_led");
    caps_label   = new QLabel (I18N_NOOP("C"),this);
    caps_led     = new KLed(this,"caps_led");
    scroll_label = new QLabel (I18N_NOOP("S"),this);
    scroll_led   = new KLed(this,"scroll_led");
    
    QFont *f1 = new QFont("Helvetica", 8); 
    
    num_label   ->setFont(*f1);
    caps_label  ->setFont(*f1);
    scroll_label->setFont(*f1);
    
    config_layout(LEFT);
    
    resize(32, 32);
    setUpdatesEnabled(true);
    show();
    
}

dockwidget::~dockwidget(){}

void dockwidget::mousePressEvent(QMouseEvent *Event)
{
    emit mouseClick(Event);
}

void dockwidget::set_LEDS (int state)
{
    ledstate = state;
    if ((ledstate & 1) == 1)
	caps_led->on();
    else
	caps_led->off();
    
    if ((ledstate & 2) == 2)
	num_led->on();
    else
	num_led->off();

    if ((ledstate & 4) == 4)
	scroll_led->on();
    else
	scroll_led->off();
}


void dockwidget::config_numled(KLed::Shape s,QColor col)
{
    num_led->setShape(s);
    num_led->setColor(col);
}

void dockwidget::config_capsled(KLed::Shape s,QColor col)
{
    caps_led->setShape(s);
    caps_led->setColor(col);
}

void dockwidget::config_scrollled(KLed::Shape s,QColor col)
{
    scroll_led->setShape(s);
    scroll_led->setColor(col);
}

void dockwidget::config_layout(uint labpos)
{
    switch (labpos) {
	case LEFT:
	    num_label   ->setGeometry(0,0,8,8);
	    caps_label  ->setGeometry(0,8,8,8);
	    scroll_label->setGeometry(0,16,8,8);
    
	    num_led   ->setGeometry(14,0,9,8);
	    caps_led  ->setGeometry(14,8,9,8);
	    scroll_led->setGeometry(14,16,9,8);
	    break;
	case RIGHT:
	    num_led   ->setGeometry(0,0,8,8);
	    caps_led  ->setGeometry(0,8,8,8);
	    scroll_led->setGeometry(0,16,8,8);
    
	    num_label   ->setGeometry(14,0,9,8);
	    caps_label  ->setGeometry(14,8,9,8);
	    scroll_label->setGeometry(14,16,9,8);
	    break;
	case TOP:
	    num_label   ->setGeometry(0,0,8,8);
	    caps_label  ->setGeometry(8,0,8,8);
	    scroll_label->setGeometry(16,0,8,8);
    
	    num_led   ->setGeometry(0,14,9,8);
	    caps_led  ->setGeometry(8,14,9,8);
	    scroll_led->setGeometry(16,14,9,8);
	    break;
	case BOTTOM:
	    num_led   ->setGeometry(0,0,8,8);
	    caps_led  ->setGeometry(8,0,8,8);
	    scroll_led->setGeometry(16,0,8,8);
    
	    num_label   ->setGeometry(0,14,9,8);
	    caps_label  ->setGeometry(8,14,9,8);
	    scroll_label->setGeometry(16,14,9,8);
	    break;
    }
}


void dockwidget::config_background(uint bg_typ, QColor bg_col)
{
    if (bg_typ==0) {
	// set background mode 
	setBackgroundMode(QWidget::PaletteBackground);
	num_label->setBackgroundMode(QWidget::PaletteBackground);
	caps_label->setBackgroundMode(QWidget::PaletteBackground);
	scroll_label->setBackgroundMode(QWidget::PaletteBackground);
    } else {
	setBackgroundColor(bg_col);
	num_label->setBackgroundColor(bg_col);
	caps_label->setBackgroundColor(bg_col);
	scroll_label->setBackgroundColor(bg_col);
    }
}

void dockwidget::config_font(QFont font)
{
    num_label->setFont(font);
    caps_label->setFont(font);
    scroll_label->setFont(font);
}
