/* 
    kleds for KDE2/KDE3

    Copyright (C) 2001, 2002, 2003 Hans Matzen (hans@hansmatzen.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __KLEDS_H 
#define __KLEDS_H 

extern "C" {
#include <stdio.h> 
}

#include <qlabel.h> 
#include <qfont.h>
#include <qsize.h>
#include <qcolor.h>
#include <qmap.h>

#include <kdockwidget.h>
#include <kmainwindow.h>
#include <kconfig.h>
#include <kled.h>
#include <dcopobject.h>

#include "dockwidget.h"
#include "dcop_if_kleds.h"

/* Label Directions */
#define LEFT   0
#define RIGHT  1
#define TOP    2
#define BOTTOM 3

/* special keys from logitech keyboard */
#define KLEDS_KEY_STANDBY 223
#define KLEDS_KEY_MUTE    160
#define KLEDS_KEY_LOWER   174
#define KLEDS_KEY_RAISE   176
#define KLEDS_KEY_PLAY    162
#define KLEDS_KEY_STOP    164
#define KLEDS_KEY_PREV    144
#define KLEDS_KEY_NEXT    153
#define KLEDS_KEY_MAIL    236
#define KLEDS_KEY_SEARCH  229
#define KLEDS_KEY_START   230
#define KLEDS_KEY_HOME    178

/* kleds main class */
class KLeds : public KMainWindow, 
	      virtual public dcop_if_kleds 
{
    Q_OBJECT

 public: 
    KLeds (QWidget *parent, const char *name); 
    ~KLeds();

 private:
    dockwidget *dockw;
    KConfig *config;
    int daemon_pid;

    /* vars for layout config */
    KLed::Shape          num_shape, caps_shape, scroll_shape;
    QColor               num_color, caps_color, scroll_color, bg_color;
    uint                 label_position, bg_typ;
    QFont                font;
    /* vars for specialkey config */
    typedef QMap<int,QString> KeyMap;
    KeyMap               special_key_cmds;

 protected:
    void preferences();    
    void read_config();
    void write_config();
    void config_dockwidget();

 public slots:
    void state_changed(int state);
    void get_pid(int pid);
    void specialkey_pressed(int keycode);

 private slots:
    void activateMenu(QMouseEvent *);
}; 
#endif 






